from django.views.generic import ListView
from json_utils.shortcuts import response_json
from querystring_parser import parser
from django.core.exceptions import FieldError
from django.db.models import Q

class KendoListProviderView(ListView):
	def _build_filters(self, filters, django_filters):
		
		for filter_id in filters:
			filter = filters[filter_id]
			if(filter.has_key('field') and filter.has_key('operator') and filter.has_key('value')):
				if(filter['operator'] == 'startswith' or filter['operator'] == 'endswith'):
					filter['operator'] = 'i'+filter['operator']

				django_filters[filter['field']+'__'+filter['operator']] = filter['value']
		return django_filters

	def _build_sorts(self, sorts, django_sorts, append_default_sorting=True):
		for sort_id in sorts:
			sort = sorts[sort_id]
			if(sort.has_key('field') and sort.has_key('dir')):
				if(sort['dir'].lower() == 'desc'):
					django_sorts.append('-%s' % sort['field'])
				else:
					django_sorts.append(sort['field'])
		if(len(django_sorts) == 0):
			django_sorts.append('id')
		return django_sorts

	def _build_groups(self, groups, django_groups):
		return self._build_sorts(groups, django_groups, False)
		

	def get(self, request, **kwargs):
		arguments = parser.parse(request.GET.urlencode())
		print arguments

		take = arguments.get('take', 10)
		skip = arguments.get('skip', 0)
		total = skip+take
		filter_arg = dict()
		sort_arg = list()
		filter_logic = 'and'

		if(arguments.has_key('filter') and arguments['filter'].has_key('filters')):
			filter_arg = self._build_filters(arguments['filter']['filters'], filter_arg)
			filter_logic = arguments['filter']['logic'].upper()

		if(arguments.has_key('group')):
			sort_arg = self._build_sorts(arguments['group'], sort_arg)

		if(arguments.has_key('sort')):
			sort_arg = self._build_sorts(arguments['sort'], sort_arg)
		
		print sort_arg
		output = dict()

		try:
			filters = Q(**filter_arg)
			filters.connector = filter_logic
			self.queryset = self.model.objects.filter(filters).order_by(*sort_arg)[skip:total]
			output = {'result':1, 'count':self.model.objects.count(), 'payload':self.get_queryset()}
		except FieldError:
			output = {'result':0, 'error':'Invalid request. Tried to filter or sort using invalid field.'}

		return response_json(request, output)
 



from django.test import TestCase
from django.test.client import RequestFactory
from django.db import models
from kendoui_backend.views import KendoListProviderView
import string
import random
import json

class DummyModel(models.Model):
	name = models.CharField(max_length=128)
	number = models.PositiveIntegerField()
	description = models.TextField()

	def __unicode__(self):
		return self.name


class KendoUITest(TestCase):
	
	def setUp(self):
		self.factory = RequestFactory()
		self.view = KendoListProviderView.as_view(model=DummyModel)
	
	def test_empty(self):
		"""
		Test if data provider yields expected result for an empty request
		"""
		request = self.factory.get('/', HTTP_ACCEPT_ENCODING='application/json')
		response = self.view(request)

		response = self.view(request)
		json_response = json.loads(response.content)
		self.assertEquals(json_response['result'], 1)
		self.assertTrue(json_response.has_key('payload'))
		self.assertEqual(len(json_response['payload']), 0)

	def test_filter_simple(self):
		"""
		Test if data provider correctly applies AND filters
		"""
		for i in range(10):
			DummyModel.objects.create(
			name="%idummy%i" % ((i%3), i),
			number = i,
			description="Some Dummy Description"
		)

		#Take up to 5 items, filter dataset for results where name starts with '1du'
		request = self.factory.get(
			'/?take=5&skip=0&page=1&pageSize=5&filter[logic]=and&filter[filters][0][field]=name&filter[filters][0][operator]=startswith&filter[filters][0][value]=1du',
			HTTP_ACCEPT_ENCODING='application/json'
		)

		response = self.view(request)
		json_response = json.loads(response.content)
		self.assertEquals(json_response['result'], 1)
		self.assertTrue(json_response.has_key('payload'))
		self.assertLessEqual(len(json_response['payload']), 5)

		for item in json_response['payload']:
				self.assertEqual(item['fields']['name'].lower()[:3], '1du')

	def test_filter_with_or_logic(self):	
		"""
		Test if data provider correctly applies OR filters
		"""
		for i in range(10):
			DummyModel.objects.create(
			name="%idummy%i" % ((i%5), i),
			number = i,
			description="Some Dummy Description"
		)

		#Take up to 5 items, filter dataset for results where name start with "1du" or number is greater than 8
		request = self.factory.get(
			'/?take=5&skip=0&page=1&pageSize=5&filter[logic]=or&filter[filters][0][field]=name&filter[filters][0][operator]=startswith&filter[filters][0][value]=1du&filter[filters][1][field]=number&filter[filters][1][operator]=gt&filter[filters][1][value]=8',
			HTTP_ACCEPT_ENCODING='application/json'
		)

		response = self.view(request)
		json_response = json.loads(response.content)
		self.assertEquals(json_response['result'], 1)
		self.assertLessEqual(len(json_response['payload']), 5)

		for item in json_response['payload']:
			if(item['fields']['name'].lower()[:3] == '1du'):
				self.assertEqual(item['fields']['name'].lower()[:3], '1du')
			elif(item['fields']['number']>8):
				self.assertGreater(item['fields']['number'], 8)
			else:
				self.fail()


	def test_sort(self):
		"""
		Test if data provider correctly sorts data.
		"""

		for i in range(10):
			DummyModel.objects.create(
			name=''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(10)),
			number = i,
			description="Some Dummy Description"
		)
		
		#Take 5 items and sort them by name ascending
		request = self.factory.get(
			'/?take=5&skip=0&page=1&pageSize=5&sort[0][field]=name&sort[0][dir]=asc',
			HTTP_ACCEPT_ENCODING='application/json'
		)

		response = self.view(request)
		json_response = json.loads(response.content)

		self.assertEquals(json_response['result'], 1)
		self.assertTrue(json_response.has_key('payload'))
		self.assertEqual(len(json_response['payload']), 5)

		last_item = None
		for item in json_response['payload']:
			if(last_item):
				self.assertGreaterEqual(item['fields']['name'], last_item)
			last_item = item['fields']['name']



#!/usr/bin/env python

from distutils.core import setup

setup(	name='django-kendoui-backend',
		version='0.1',
		description='Kendo UI DataSource backend',
		author='Tom Najdek',
		author_email='tom@doppnet.com',
		license='MIT',
		url='https://github.com/tnajdek/django-kendoui-backend',
		packages=['kendoui_backend'],
		install_requires=[
			'django>=1.3.0',
			'json_utils>=0.2',
			'querystring_parser>=1.0'
		],
)